/*
 * main.c
 *
 * Let LED0, LED1, LED2 and LED3 blink with different frequencies using multitasking.
 *
 * This example is an advanced training material for experienced students.
 *
 * It is not necessary to implement one function per task. A function can be used by
 * several tasks. Parameters tells it what to do. The parameters have to be passed by
 * a pointer. If more than one parameter is needed a struct is used.
 *
 * Created: 2017-11-29
 * Modified: 2018-07-23
 *  Author: Wolfgang Neff
 */

#include <avr/io.h>
#include <FreeRTOS.h>
#include <task.h>

#include "board.h"

#define TASK_PRIORITY (tskIDLE_PRIORITY+1)

// We want to pass two bits of information:
// - The bitmap of the LED.
// - The ticks to delay.
typedef struct {
	uint8_t led;
	TickType_t delay;
} led_params_t;

// We define the parameters for each LED.
led_params_t led0_params = { LED0, pdMS_TO_TICKS(100) };
led_params_t led1_params = { LED1, pdMS_TO_TICKS(200) };
led_params_t led2_params = { LED2, pdMS_TO_TICKS(300) };
led_params_t led3_params = { LED3, pdMS_TO_TICKS(400) };

void vLEDTask(void* pvParameters)
{
	// Parameters are void*. This means they are anonymous.
	// We have to give them the correct data type in order
	// to be able to access the components of the struct.
	led_params_t* params = (led_params_t*)pvParameters;
	// Initialize the hardware of the task.
	LED_PORT |= (1 << params->led);
	LED_DDR |= (1 << params->led);
	// Super loop of the task.
	while (1) {
		// Toggle the LED.
		LED_PORT ^= (1 << params->led);
		// Wait a bit.
		vTaskDelay(params->delay);
	}
}

int main(void)
{
	// Create the tasks.
	xTaskCreate(vLEDTask, "LED0Task", configMINIMAL_STACK_SIZE, &led0_params, TASK_PRIORITY, NULL);
	xTaskCreate(vLEDTask, "LED1Task", configMINIMAL_STACK_SIZE, &led1_params, TASK_PRIORITY, NULL);
	xTaskCreate(vLEDTask, "LED2Task", configMINIMAL_STACK_SIZE, &led2_params, TASK_PRIORITY, NULL);
	xTaskCreate(vLEDTask, "LED3Task", configMINIMAL_STACK_SIZE, &led3_params, TASK_PRIORITY, NULL);
	
	// Start the scheduler.
	vTaskStartScheduler();
}
