/*
 * main.c
 *
 * A task lets LED1 blink. The task can be suspended by button 1 and resumed by button 2.
 *
 * This example is an advanced training material for experienced students.
 *
 * Tasks can be created, killed, suspended and resumed. This example program shows how a
 * how a task is suspended and resumed. We never kill tasks. We use a very simple memory
 * manager and are not able to release the resources used by a task. Note: do not use
 * suspend and resume to synchronize tasks. Use binary semaphores instead.
 *
 * Please see www.freertos.org for more information.
 * xTaskCreate: FreeRTOS/API Reference/Task Creation
 * vTaskResume: FreeRTOS/API Reference/Task Control
 * vTaskSuspend: FreeRTOS/API Reference/Task Control
 *
 * Created: 2017-11-29
 * Modified: 2018-07-23
 *  Author: Wolfgang Neff
 */

#include <avr/io.h>
#include <FreeRTOS.h>
#include <task.h>

#include "board.h"

#define LED_TASK_PRIORITY (tskIDLE_PRIORITY+2)
#define LED_DELAY pdMS_TO_TICKS(500)

#define BUTTON_TASK_PRIORITY (tskIDLE_PRIORITY+1)
#define BUTTON_TASK_PERIOD pdMS_TO_TICKS(100)

// We need a task handle to be able to suspend and resume it.
TaskHandle_t xLEDTaskHandle;

void vLEDTask(void* pvParameters)
{
	// Initialize the LED.
	LED_DDR |= (1 << LED1);
	LED_PORT |= (1 << LED1);
	// Super loop of the task.
	while (1) {
		LED_PORT ^= (1 << LED1);
		vTaskDelay(LED_DELAY);
	}
}

void vButtonTask(void* pvParameters)
{
	// Initialize the push-buttons.
	BUTTON_PORT |= ((1 << BUTTON1) | (1 << BUTTON2));
	// Super loop of the task.
	while (1) {
		if (!(BUTTON_PIN & (1 << BUTTON1))) {
			vTaskResume(xLEDTaskHandle);
		}
		if (!(BUTTON_PIN & (1 << BUTTON2))) {
			vTaskSuspend(xLEDTaskHandle);
		}
		vTaskDelay(BUTTON_TASK_PERIOD);
	}
}

int main(void)
{
	// Create the tasks.
	xTaskCreate(vLEDTask, "LEDTask", configMINIMAL_STACK_SIZE, NULL, LED_TASK_PRIORITY, &xLEDTaskHandle);
	xTaskCreate(vButtonTask, "ButtonTask", configMINIMAL_STACK_SIZE, NULL, BUTTON_TASK_PRIORITY, NULL);

    // When creating a task it has state ready. We have to suspend it before
	// the scheduler starts if we do not want that it starts automatically. 
    vTaskSuspend(xLEDTaskHandle);

	// Start the scheduler.
	vTaskStartScheduler();
}
