/*
 * main.c
 *
 * This example is an advanced training material for experienced students.
 *
 * Message queues are not limited to simple data types. You can use a struct
 * to pass more complex data. This will be demonstrated here. A button task
 * checks button 0, 1 and 2 and lets LED 1 blink 1, 2 or 3 times.
 *
 *  Created: 2017-12-01
 * Modified: 2018-07-23
 *   Author: Wolfgang Neff
 */ 

#include <avr/io.h>
#include <FreeRTOS.h>
#include <queue.h>
#include <task.h>

#include "board.h"
#include "switch.h"

#define CAPACITY 8

#define LED_TASK_PRIORITY (tskIDLE_PRIORITY+2)
#define LED_DELAY pdMS_TO_TICKS(250)

#define BUTTON_TASK_PRIORITY (tskIDLE_PRIORITY+1)
#define BUTTON_TASK_DELAY pdMS_TO_TICKS(25)

// Define the data type of the message.
typedef struct {
	uint8_t led;     // The bitmap of the LED to blink.
	uint8_t blinks;  // The number of flashes.
} message_t;

// Declare the message queue.
QueueHandle_t xQueue;

// This task reads the message queue and blinks accordingly.
void vLEDTask(void* pvParameters)
{
	// Declare the message buffer.
    message_t message;
	// Initialize the LEDs.
    LED_DDR |= (1 << LED1);
    LED_PORT |= (1 << LED1);
	while (1) {
		// Await the message and let the selected led blink.
		xQueueReceive(xQueue, &message, portMAX_DELAY);
		for (uint8_t i=0; i<2*message.blinks; i++) {
			LED_PORT ^= (1 << message.led);
			vTaskDelay(LED_DELAY);
		}
	}
}

// Check the buttons and send an appropriate message.
void vButtonTask(void* pvParameters)
{
	// Declare the message buffer.
    message_t message;
    // Initialize the push-buttons
	BUTTON_PORT |= (1 << BUTTON0) | (1 << BUTTON1) | (1 << BUTTON2);
	while (1) {
		// Check the push-buttons and send the corresponding message.
		if (SWITCH0_PRESSED()) {
			message.led = LED1;
			message.blinks = 1;
			xQueueSend(xQueue, &message, portMAX_DELAY);
		}
		if (SWITCH1_PRESSED()) {
			message.led = LED1;
			message.blinks = 2;
			xQueueSend(xQueue, &message, portMAX_DELAY);
		}
		if (SWITCH2_PRESSED()) {
			message.led = LED1;
			message.blinks = 3;
			xQueueSend(xQueue, &message, portMAX_DELAY);
		}
		// Wait a bit before checking the button again.
		vTaskDelay(BUTTON_TASK_DELAY);
	}
}

int main(void)
{
	// Create the message queue.
	xQueue = xQueueCreate(CAPACITY, sizeof(message_t));

	// Create the tasks.
	xTaskCreate(vLEDTask, "LEDTask", configMINIMAL_STACK_SIZE, NULL, LED_TASK_PRIORITY, NULL);
	xTaskCreate(vButtonTask, "ButtonTask", configMINIMAL_STACK_SIZE, NULL, BUTTON_TASK_PRIORITY, NULL);

	// Start the scheduler.
    vTaskStartScheduler();
}
